#include "B4RDefines.h"

namespace B4R
{

    void B4RRCSwitch::Initialize()
    {
       rcs = new (backend) RCSwitch();
    };

    void B4RRCSwitch::looper(void* b) 
    {
       B4RRCSwitch* me = (B4RRCSwitch*)b;
       if (me->rcs->available()) { 
          me->DataAvailableSub();}
    };

    void B4RRCSwitch::enableReceive(SubVoidVoid DataAvailableSub, int interrupt)
    {
       this->DataAvailableSub = DataAvailableSub; 
       FunctionUnion fu; 
       fu.PollerFunction = looper;
       pnode.functionUnion = fu;
       pnode.tag = this; 
       if (pnode.next == NULL) 
          pollers.add(&pnode); 
       rcs->enableReceive(interrupt);
    };
    
    void B4RRCSwitch::switchOn(int nGroupNumber, int nSwitchNumber)
    {
       rcs->switchOn(nGroupNumber, nSwitchNumber);
    };

    void B4RRCSwitch::switchOff(int nGroupNumber, int nSwitchNumber)
    {
       rcs->switchOff(nGroupNumber, nSwitchNumber);
    };

    void B4RRCSwitch::switchOn1(B4RString* sGroup, int nSwitchNumber)
    {
       rcs->switchOn(sGroup->data, nSwitchNumber);
    };

    void B4RRCSwitch::switchOff1(B4RString* sGroup, int nSwitchNumber)
    {
       rcs->switchOff(sGroup->data, nSwitchNumber);
    };

    void B4RRCSwitch::switchOn2(char sFamily, int nGroup, int nDevice)
    {
       rcs->switchOn(sFamily, nGroup, nDevice);
    };

    void B4RRCSwitch::switchOff2(char sFamily, int nGroup, int nDevice)
    {
       rcs->switchOff(sFamily, nGroup, nDevice);
    };

    void B4RRCSwitch::switchOn3(B4RString* sGroup, B4RString* sDevice)
    {
       rcs->switchOn(sGroup->data, sDevice->data);
    };

    void B4RRCSwitch::switchOff3(B4RString* sGroup, B4RString* sDevice)
    {
       rcs->switchOff(sGroup->data, sDevice->data);
    };

    void B4RRCSwitch::switchOn4(char sGroup, int nDevice)
    {
       rcs->switchOn(sGroup, nDevice);
    };

    void B4RRCSwitch::switchOff4(char sGroup, int nDevice)
    {
       rcs->switchOff(sGroup, nDevice);
    };

    void B4RRCSwitch::sendTriState(B4RString* sCodeWord)
    {
       rcs->sendTriState(sCodeWord->data);
    };

    void B4RRCSwitch::send(ulong code, uint length)
    {
       rcs->send(code, length);
    };

    void B4RRCSwitch::send1(B4RString* sCodeWord)
    {
       rcs->send(sCodeWord->data);
    };

    void B4RRCSwitch::disableReceive()
    {
       rcs->disableReceive();
    };

    void B4RRCSwitch::resetAvailable()
    {
       rcs->resetAvailable();
    };

    ulong B4RRCSwitch::getReceivedValue()
    {
       return rcs->getReceivedValue();
    };

    uint B4RRCSwitch::getReceivedBitlength()
    {
       return rcs->getReceivedBitlength();
    };

    uint B4RRCSwitch::getReceivedDelay()
    {
       return rcs->getReceivedDelay();
    };

    uint B4RRCSwitch::getReceivedProtocol()
    {
       return rcs->getReceivedProtocol();
    };

    ArrayUInt* B4RRCSwitch::getReceivedRawdata()
    {
        uint* raw = rcs->getReceivedRawdata(); 
        unsigned int len = 67; //raw.size(); 
        ArrayByte* arr = CreateStackMemoryObject(ArrayByte);
        arr->data = (StackMemory::buffer + StackMemory::cp);
        for (int i = 0;i < len; i++) {
            StackMemory::buffer[StackMemory::cp++] = raw[i] & 0xFF;        
            StackMemory::buffer[StackMemory::cp++] = (raw[i] >> 8) & 0xFF;  
        }
        arr->length=len;
        return arr;
    };
                                                                
    void B4RRCSwitch::enableTransmit(int nTransmitterPin)
    {
       rcs->enableTransmit(nTransmitterPin);
    };

    void B4RRCSwitch::disableTransmit()
    {
       rcs->disableTransmit();
    };

    void B4RRCSwitch::setPulseLength(int nPulseLength)
    {
       rcs->setPulseLength(nPulseLength);
    };

    void B4RRCSwitch::setRepeatTransmit(int nRepeatTransmit)
    {
       rcs->setRepeatTransmit(nRepeatTransmit);
    };

    void B4RRCSwitch::setReceiveTolerance(int nPercent)
    {
       rcs->setReceiveTolerance(nPercent);
    };
  
    void B4RRCSwitch::ProtocolSet(uint16_t pulseLength, uint8_t syncFactor_high, uint8_t syncFactor_low, uint8_t zero_high, uint8_t zero_low, uint8_t one_high, int8_t one_low, bool invertedSignal )
    {
       	this->protocol.pulseLength = pulseLength;
       	this->protocol.syncFactor.high = syncFactor_high;
       	this->protocol.syncFactor.low = syncFactor_low;
       	this->protocol.zero.high = zero_high;
      	this->protocol.zero.low = zero_low;
      	this->protocol.one.high = one_high;
      	this->protocol.one.low = one_high;
      	this->protocol.invertedSignal = invertedSignal;                                                                          
       rcs->setProtocol(protocol);
   };

    void B4RRCSwitch::setProtocol1(int nProtocol)
    {
       rcs->setProtocol(nProtocol);
    };

    void B4RRCSwitch::setProtocol2(int nProtocol, int nPulseLength)
    {
       rcs->setProtocol(nProtocol, nPulseLength);
    };

}
